import discord
import discord.ext.commands as commands
from discord.utils import find

from src.config_dict import ConfigDict


class Configuration(commands.Cog):
    def __init__(self, bot, config: ConfigDict):
        self.bot = bot
        self.config = config

    @commands.command()
    async def channel(self, message: discord.Message):
        """Set the channel to be used by bot"""
        channel = self.bot.get_channel(message.content)

        with self.bot.lock:
            self.bot.config["notification_channel_id"] = channel.id

    @commands.command()
    async def min_players(self, message: discord.Message):
        """Set minimum amount of players required to trigger notification"""
        try:
            min_pop = int(message.content, 10)

            with self.bot.lock:
                self.config["min_players"] = min_pop
        except ValueError:
            await message.reply("Provided value was not a valid number.")

    @commands.command()
    async def min_ping(self, message: discord.Message):
        """Set minimum amount of players required to trigger notification"""
        try:
            min_pop = int(message.content, 10)

            with self.bot.lock:
                self.config["min_ping_players"] = min_pop
        except ValueError:
            await message.reply("Provided value was not a valid number.")

    @commands.command()
    async def timeout(self, message: discord.Message):
        """Set minimum amount of players required to trigger notification"""
        try:
            min_pop = int(message.content, 10)

            with self.bot.lock:
                self.config["ping_timeout"] = min_pop
        except ValueError:
            await message.reply("Provided value was not a valid number.")

    @commands.command()
    async def role(self, message: discord.Message):
        role = find(lambda r: r.name == message.content, message.guild.roles)

        with self.bot.lock:
            self.config["role_id"] = role.id

