import discord.ext.commands as commands
from discord.ext import tasks

from src.services.notification_service import NotificationService
from src.services.notification_type import NotificationType


class Notification(commands.Cog):
    def __init__(self, bot, config, server_service):
        self.bot = bot
        self.config = config
        self.service = NotificationService(config, server_service)

        self.send_server_notification.start()

    @tasks.loop(seconds=300)
    async def send_server_notification(self, is_update_forced=False):
        channel = self.bot.get_channel(self.config["notification_channel_id"])
        role = channel.guild.get_role(self.config["role_id"])

        servers = self.service.fetch_populated_servers_info()
        msg_content = self.service.create_msg_content(servers, role.mention)
        last_message = await channel.fetch_message(channel.last_message_id)
        notification_type = self.service.get_notification_type(last_message.author.id, self.bot.user.id,
                                                               servers, is_update_forced)
        if notification_type == NotificationType.NoNotification:
            return

        if notification_type == NotificationType.New:
            return await channel.send(msg_content)

        if notification_type == NotificationType.Edit:
            # Quick hack to keep ping in messages with them, otherwise it deletes the ping from discord notifications
            if last_message.content.find(role.mention) != -1:
                msg_content = f"{role.mention}\n" + msg_content

            return await last_message.edit(content=msg_content)

        if notification_type == NotificationType.Forced:
            # Forced notifications always will be new messages
            return await channel.send(msg_content)

    @send_server_notification.before_loop
    async def before_fetch_task(self):
        await self.bot.wait_until_ready()

    @commands.command()
    async def update(self, ctx):
        """Force active server check"""
        await self.send_server_notification(is_update_forced=True)
        self.send_server_notification.restart()
