import discord
import discord.ext.commands as commands


# TODO Rework this to reduce duplication, allow for dynamically changing the message id
class Roles(commands.Cog):
    def __init__(self, bot: discord.ext.commands.Bot, role_id, message_id, channel_id):
        self.bot = bot
        self.role_id = role_id
        self.message_id = message_id
        self.channel_id = channel_id
        self.emoji = discord.PartialEmoji(name="✅")

    @commands.Cog.listener()
    async def on_raw_reaction_add(self, payload: discord.RawReactionActionEvent):
        role_and_guild = self.get_role_and_guild(payload)
        if role_and_guild[0] is None or role_and_guild[1] is None:
            return

        try:
            # Finally, add the role.
            await payload.member.add_roles(role_and_guild[0])
        except discord.HTTPException:
            # If we want to do something in case of errors we'd do it here.
            pass

    @commands.Cog.listener()
    async def on_raw_reaction_remove(self, payload: discord.RawReactionActionEvent):
        role_and_guild = self.get_role_and_guild(payload)
        if role_and_guild[0] is None or role_and_guild[1] is None:
            return

        member = role_and_guild[1].get_member(payload.user_id)
        if member is None:
            # Make sure the member still exists and is valid.
            return

        try:
            # Finally, remove the role.
            await member.remove_roles(role_and_guild[0])
        except discord.HTTPException:
            # If we want to do something in case of errors we'd do it here.
            pass

    def get_role_and_guild(self, payload: discord.RawReactionActionEvent):
        """Returns a tuple where index 0 is Role, and index 1 is Guild"""
        # Ignore reactions given by bot
        if payload.user_id == self.bot.user.id:
            return

        # Make sure that the message the user is reacting to is the one we care about.
        if payload.message_id != self.message_id:
            return

        # The payload for `on_raw_reaction_remove` does not provide `.member`
        # so we must get the member ourselves from the payload's `.user_id`.
        guild = self.bot.get_guild(payload.guild_id)
        if guild is None:
            # Check if we're still in the guild and it's cached.
            return

        role = guild.get_role(self.role_id)
        if role is None:
            # Make sure the role still exists and is valid.
            return

        return role, guild

    @commands.Cog.listener()
    async def on_ready(self):
        channel = self.bot.get_channel(self.channel_id)
        message = await channel.fetch_message(self.message_id)
        await message.add_reaction(self.emoji)

